// Aspell check functions
// Copyright 1998 by Kevin Atkinson under the terms of the LGPL

#ifndef as_check_t__
#define as_check_t__

#include "check.hh"

#define tl template <class Itr, class ItrEnd>
#define tld template <class Itr, class ItrEnd = Itr>
#define tt Itr,ItrEnd

tl void url_skip_iterator<tt>::scan_ahead() {
  enum {slash, who_cares} prev_char = who_cares;
  enum {wbegin, wmiddle, wend} word_pos = wbegin;
  blank_out_ = false;
  int point_chars = 0;
  Itr next;
  if (cur_ == end_) return;
  for (;;) {
    if (word_pos == wend) break;
    next = cur_; ++next;
    if (next == end_ || *next == ' ' || *next == '\n' || *next == '\t')
      word_pos = wend;
    if (word_pos == wmiddle) {
      switch (*cur_) {
      case '@':
	blank_out_ = true;
	prev_char = who_cares;
	break;
      case '.':
	++point_chars;
	prev_char = who_cares;
	break;
      case '/':
	if (prev_char == slash) blank_out_ = true;
	prev_char = slash;
	break;
      default:
	prev_char = who_cares;
      }
    }
    ++cur_;
    if (word_pos == wbegin) word_pos = wmiddle;
  }
  if (point_chars > 1) blank_out_ = true;
}

#undef tl
#undef tt
#undef tld

template <class ForwardIterator, class EndIterator, class StatusFunc>
bool check (const aspell & sc, 
	    aspell_check_state<ForwardIterator, EndIterator> & state,
	    const StatusFunc & print_status)
{
  const char *alpha = sc.alpha_map();

  typedef 
    aspell_check_state<ForwardIterator, EndIterator>::skip_iterator
    skip_iterator;

  *state.word_end_ = *state.word_begin_;
  skip_iterator & i  = *state.word_end_;

  string word;
  bool   cur_is_alpha  = false;
  bool   prev_is_alpha = false;
  for (; !i.is_end(); ++i) {
    cur_is_alpha = alpha[*i];
    if (cur_is_alpha != prev_is_alpha) {
      if (prev_is_alpha) {
	if (sc.check(word)) {
	  print_status();
	} else {
	  state.save_state();
	  return true;
	} 
      } else { // cur_is_alpha
	word.resize(0);
	*state.word_begin_ = i;
      }
    }
    if (cur_is_alpha) {
      word += *i;
    } else {
      // do nothing
    }
    prev_is_alpha = cur_is_alpha;
  }
  if (cur_is_alpha) {
    if (sc.check(word)) {
      print_status();
    } else {
      state.save_state();
      return true;
    }
  }

  *state.word_begin_ = i;
  return false;
}

#endif
